//The MIT License
//
// Copyright (c) 2004 Evren Sirin
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

package org.mindswap.wsdl;

import java.io.FileNotFoundException;
import java.io.OutputStream;
import java.io.Writer;
import java.net.URI;

import org.mindswap.owl.OWLFactory;
import org.mindswap.owl.OWLKnowledgeBase;
import org.mindswap.owl.OWLOntology;
import org.mindswap.owl.OWLType;
import org.mindswap.owls.grounding.AtomicGrounding;
import org.mindswap.owls.grounding.Grounding;
import org.mindswap.owls.grounding.WSDLAtomicGrounding;
import org.mindswap.owls.process.AtomicProcess;
import org.mindswap.owls.process.Input;
import org.mindswap.owls.process.Output;
import org.mindswap.owls.profile.Profile;
import org.mindswap.owls.service.Service;
import org.mindswap.utils.URIUtils;

public class WSDLTranslator {      
    private Service service;    
    private OWLOntology ont;
    URI baseURI;

	  
    public WSDLTranslator(WSDLOperation op, String serviceURI, String prefix) {
        OWLKnowledgeBase kb = OWLFactory.createKB();
        ont = kb.createOntology();
        
        baseURI = URIUtils.createURI(serviceURI);        
        
		service = ont.createService(URIUtils.createURI(baseURI, prefix + "Service"));
		
		Profile profile = ont.createProfile(URIUtils.createURI(baseURI, prefix + "Profile"));
		AtomicProcess process = ont.createAtomicProcess(URIUtils.createURI(baseURI, prefix + "Process"));
		Grounding grounding = ont.createGrounding(URIUtils.createURI(baseURI, prefix + "Grounding"));
		WSDLAtomicGrounding ag = ont.createWSDLAtomicGrounding(URIUtils.createURI(baseURI, prefix + "AtomicProcessGrounding"));
		
		process.setLabel(process.getURI().getFragment());
		
		// set the links between structures
		service.setProfile(profile);
		service.setProcess(process);
		service.setGrounding(grounding);

		// add the WSDL details to the atomic grounding
		ag.setProcess(process);
		ag.setOperation(URI.create(op.getOperationName()));
		ag.setWSDL(op.getService().getFileURI());
		ag.setPortType(URI.create(op.getPortName()));
		ag.setInputMessage(URI.create(op.getInputMessageName()));
		ag.setOutputMessage(URI.create(op.getOutputMessageName()));

		// add the atomic process grounding to service grounding
		grounding.addGrounding(ag);	
    }
    
    public void setServiceName(String serviceName) {
        service.getProfile().setServiceName(serviceName);
    }
    
    public void setTextDescription(String textDescription) {
        service.getProfile().setTextDescription(textDescription);
        System.out.println(service.getProfile().getTextDescription());
    }

    	public void addInput(WSDLParameter param, String paramName, URI paramType, String xsltTransformation) {   //++
 //++   public void addInput(WSDLParameter param, String paramName, URI paramType) {
 //++  	public void addInput(WSDLParameter param, String paramName) {

       Profile profile = service.getProfile();
		AtomicProcess process = (AtomicProcess) service.getProcess();
		
		// create process param
		Input input = ont.createInput(URIUtils.createURI(baseURI, paramName));
		input.setLabel(paramName);
				
    	OWLType type = ont.getType(paramType); //++
    	input.setParamType(type == null ? ont.createClass(paramType) : type); //++
			
		// add the param to process and profile
		process.addInput(input);
		profile.addInput(input);
		
		AtomicGrounding grounding = process.getGrounding();
		// create grounding message map
		
		grounding.addMessageMap(input, param.getName(), xsltTransformation);//++
		grounding.addMessageMap(input, param.getName());
    }
   
    	public void addOutput(WSDLParameter param, String paramName, URI paramType, String xsltTransformation) {
 //++   public void addOutput(WSDLParameter param, String paramName, URI paramType) {
 //++   public void addOutput(WSDLParameter param, String paramName) {	
    Profile profile = service.getProfile();
		AtomicProcess process = (AtomicProcess) service.getProcess();
		System.out.println("===================param==========="+ param);
		System.out.println("===================paramName==========="+ paramName);
		System.out.println("===================paramType==========="+ paramType);
		System.out.println("=================xsltTransformation ==========="+ xsltTransformation);
		
		// create process param
		Output output = ont.createOutput(URIUtils.createURI(baseURI, paramName));
		output.setLabel(paramName);
		
    	OWLType type = ont.getType(paramType); //++
    	output.setParamType(type == null ? ont.createClass(paramType) : type);//++
    	System.out.println("==================== output ds WSDLTranslator ==========="+ output);
    	System.out.println("=================param.getName() ==========="+ param.getName());
    
		// add the param to process and profile
		process.addOutput(output);
		profile.addOutput(output);
		
		AtomicGrounding grounding = process.getGrounding();
		// create grounding message map
		grounding.addMessageMap(output, param.getName(), xsltTransformation);   
    	grounding.addMessageMap(output, param.getName());   //++
    }
	
	public void writeOWLS(Writer out) {
	    ont.write(out, baseURI);
    }
   
    public void writeOWLS(OutputStream out) {
        ont.write(out, baseURI);
    }
/*	*********************************	Methode+Main Faten 		******************************	*/
///////////// Methode ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZz    
      
    public static void main( String[] args ) throws Exception {
    	 System.out.println("*****************	Main  111	******************** ");
    	final URI baseURI = URI.create("file:///E:/addition2_f1.owl");
		String ont1 = "file:///E:/addition2_f1.owl";
		 System.out.println("*****************	Main  222	******************** ");
		final OWLOntology ont = OWLFactory.createKB().read(URI.create(ont1));
		 System.out.println("*****************	Main  222	****  read ******************** ");
		final Service service = ont.createService(URIUtils.createURI(baseURI, "Service"));
		 System.out.println("*****************	Main  222	****  service ******************** ");
		final Profile profile = ont.createProfile(URIUtils.createURI(baseURI, "Profile"));
		 System.out.println("*****************	Main  333	*****  profile *************** ");
		 System.out.println("================== profile======================="+profile.toString());
		 
		 final AtomicProcess process = ont.createAtomicProcess(URIUtils.createURI(baseURI, "Process"));
		 System.out.println("*****************	Main  333	*****  process *************** ");
			service.setProcess(process);
			System.out.println("*****************	Main  333	*****  setProcess *************** ");
		final Input input1 = ont.createInput(URIUtils.createURI(baseURI, "City"));
    	 System.out.println("*****************	Main  444	******************** ");
    	 //System.out.println("================== input1======================="+input1);
		//OWLType owlTypeCity = ont.getType(URIUtils.createURI("http://www.daml.org/services/owl-s/1.2/Concepts.owl#City"));
    	 //OWLType owlTypeCity = ont.getType(URIUtils.createURI("http://127.0.0.1/ontology/books.owl#Author"));
    	 //OWLType owlTypeCity = ont.getType(URIUtils.createURI("file:///E:/books.owl#Author"));
    	 OWLType owlTypeCity = ont.getType(URIUtils.createURI("http://www.daml.org/services/owl-s/1.2/Concepts.owl#City"));
    	 
    	 final Input input2 = ont.createInput(URIUtils.createURI(baseURI, "Period"));
         OWLType owlTypePeriod = ont.getType(URIUtils.createURI("http://www.daml.org/services/owl-s/1.2/Concepts.owl#Period"));
         System.out.println("*****************	owlTypePeriod	**********    "+ owlTypePeriod);
         input2.setParamType(owlTypePeriod);
         profile.addInput(input2);
         process.addInput(input2);

   	 
		 System.out.println("*****************	Main  555	******************** ");
		 input1.affich_faten();
		 System.out.println("*****************	Main  555 ttttt	******************** ");
		 System.out.println("*****************	owlTypeCity	**********    "+ owlTypeCity);
			
		 input1.setParamType(owlTypeCity);
		 System.out.println("*****************	Main  666	******************** ");
		profile.addInput(input1);
		 System.out.println("*****************	Main  777	******************** ");
		//process.addInput(input1);
    }
/*	*********************************   Fin				******************************	*/
    public void addOutput2(WSDLParameter param, String paramName, URI paramType, String xsltTransformation) {
    	    Profile profile = service.getProfile();
    			AtomicProcess process = (AtomicProcess) service.getProcess();
    			System.out.println("===================param==========="+ param);
    			System.out.println("===================paramName==========="+ paramName);
    			System.out.println("===================paramType==========="+ paramType);
    			System.out.println("=================xsltTransformation ==========="+ xsltTransformation);
    			
    			// create process param
    			Output output = ont.createOutput(URIUtils.createURI(baseURI, paramName));
    			output.setLabel(paramName);
    			
    	    	OWLType type = ont.getType(paramType); //++
    	    	output.setParamType(type == null ? ont.createClass(paramType) : type);//++
    	    	System.out.println("==================== output ds WSDLTranslator ==========="+ output);
    	    	System.out.println("=================param.getName() ==========="+ param.getName());
    	    
    			// add the param to process and profile
    			process.addOutput(output);
    			profile.addOutput(output);
    			
    			AtomicGrounding grounding = process.getGrounding();
    			// create grounding message map
    			grounding.addMessageMap(output, param.getName(), xsltTransformation);   
    	    	grounding.addMessageMap(output, param.getName());   //++
    	    }
    		

}
